package com.gic.cloud.data.hook.service.impl;

import cn.medubi.client.utils.LogPak;
import com.alibaba.fastjson.JSON;
import com.ctrip.framework.apollo.Config;
import com.ctrip.framework.apollo.ConfigService;
import com.gic.cloud.data.hook.api.dto.*;
import com.gic.cloud.data.hook.api.dto.DataDesensiType;
import com.gic.cloud.data.hook.api.entity.*;
import com.gic.cloud.data.hook.api.service.IDownloadTaskService;
import com.gic.cloud.data.hook.api.service.IFreeQueryService;
import com.gic.cloud.data.hook.service.*;
import com.gic.cloud.data.hook.service.dao.FreeQueryRecordDao;
import com.gic.cloud.data.hook.service.dao.FreeQuerySourceDao;
import com.gic.cloud.data.hook.service.entity.CsvDataFilterMode;
import com.gic.cloud.data.hook.service.entity.CsvResultSetHelper;
import com.gic.qcloud.BucketNameEnum;
import com.gic.qcloud.FileUploadUtil;
import com.google.common.collect.Lists;
import com.opencsv.CSVWriter;
import com.opencsv.ResultSetHelper;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.concurrent.BasicThreadFactory;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.io.*;
import java.nio.charset.Charset;
import java.sql.*;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.Date;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/** 自定义查询服务接口实现
 * @author Sam
 */
public class FreeQueryServiceImpl implements IFreeQueryService {

	/** 日志类 */
	private static LogPak log = new LogPak(FreeQueryServiceImpl.class);

	/** csv / xls 下载目录 */
	public static final String SAVE_FOLDER = "/usr/local/data-hook-file";


	@Autowired
	IDownloadTaskService downloadTaskService;

	/** 获取指定脱敏类型的过滤字段集合
	 * @param desensiType
	 * @return
	 */
	public static List<String> getFieldsFilters(Integer desensiType) {
		if (desensiType == DataDesensiType.PHONE_ONLY) {
			return FilterFieldUtils.FILTERS_PHONE_ONLY;
		} else return FilterFieldUtils.FILTERS_PHONE_AND_CARD;
	}

	/** 是否为脱敏字段 */
	public static boolean isFilterFields(Integer desensiType, String fieldName) {
		List<String> fieldsFilter = getFieldsFilters(desensiType);
		log.debug("isFilterFields:", JSON.toJSONString(fieldsFilter) + "-" + desensiType +"-"+ fieldName);
		for (String filter : fieldsFilter) {
			if (fieldName.contains(filter)) return true;
		} // FOR OVER
		return false;
	}

	/** 自定义查询关联的下载条件列表 */
	protected List<FreeQueryTaskCondition> taskConditions = Lists.newArrayList();

	/** 自定义查询记录 DAO */
	@Autowired
	private FreeQueryRecordDao freeQueryRecordDao;

	/** 自定义查询数据源 DAO */
	@Autowired
	private FreeQuerySourceDao freeQuerySourceDao;

	/** 构造函数 */
	private FreeQueryServiceImpl() {
		log.debug("construct", "准备初始化 FreeQuery 查询服务");
		this.runDownloadTask(3); // 3秒轮训
		this.runApplyTask(5); // 每5秒钟进行任务状态检测
	}

	@Override
	public List<FreeQueryTable> getFreeQueryTableList() {
		log.debug("getFreeQueryTableList", "getFreeQueryTableList");
		return MysqlHelper.getFreeQueryTableList();
	}

	@Override
	public Integer createFreeQueryRecord(FreeQueryRecord record) {
		return this.freeQueryRecordDao.createFreeQueryRecord(record);
	}

	@Override
	public Integer updateFreeQueryRecord(FreeQueryRecord record) {
		return this.freeQueryRecordDao.updateFreeQueryRecord(record);
	}

	@Override
	public Integer deleteFreeQueryRecord(String recordId) {
		return this.freeQueryRecordDao.deleteFreeQueryRecord(recordId);
	}

	@Override
	public FreeQueryRecord getFreeQueryRecordByUserIdAndRecordId(String userId, String recordId) {
		return this.freeQueryRecordDao.getFreeQueryRecordByUserIdAndRecordId(userId, recordId);
	}

	@Override
	public List<FreeQueryRecord> getFreeQueryRecordListByUserId(String userId, String fuzzy) {
		return this.freeQueryRecordDao.getFreeQueryRecordListByUserId(userId, fuzzy);
	}


	/** 修订 SQL 语句（将 dmwl_test 标记替换为实际库）
	 * @param sql
	 * @return
	 */
//	public String prefixSQL(String enterpriseId, String sql) {
//		FreeQuerySource source = this.freeQuerySourceDao.getFreeQuerySourceByEnterpriseId(enterpriseId);
//		return sql.replace("dmwl_test", "gic_data_" + source.getEnterpriseId());
//	}


	/** 获取指定表的所有字段
	 * @param tableName
	 * @return
	 */
	public List<String> getColumnsByTable(String tableName) {
		List<String> result = Lists.newArrayList();
		Connection conn = MysqlHelper.getFreeQueryConnection("6c68a26e6f0d11e59d0818c58a146ef4");
		if (conn != null) {
			try {
				log.debug("getColumnsByTable", "准备查询表信息 " + tableName);
				Statement stat = conn.createStatement();
				ResultSet rs = stat.executeQuery("select column_name  from information_schema.columns  where table_schema =  'amx' and table_name = '" + tableName + "' ");
				while (rs.next()) {
					result.add(rs.getString(1));
					//log.debug("getColumnsByTable", "获取字段：" + rs.getString(1));
				}
			} catch (Exception ex) {
				ex.printStackTrace();
			} finally {
				try {
					conn.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		} // IF OVER
		// 返回结果
		return result;
	}

	@Override
	public FreeQuerySource getFreeQuerySource(String enterpriseId) {
		return MysqlHelper.instance.getFreeQuerySource(enterpriseId);
	}

	@Override
	public Integer getFreeQueryCount(String sql, String enterpriseId) {
		//sql = prefixSQL(enterpriseId, sql);
		Integer result = -1;
		log.debug("getFreeQueryCount", "Count 自定义查询来自 " + enterpriseId);
		log.debug("getFreeQueryCount", "自定义查询SQL：" + sql);
		Connection conn = MysqlHelper.getFreeQueryConnection(enterpriseId);
		if (conn != null) {
			try {
				Statement stat = conn.createStatement();
				ResultSet rs = stat.executeQuery("SELECT COUNT(1) FROM (" + sql + ") AS tmpTable");
				if (rs.next()) {
					result = rs.getInt(1);
					log.debug("getFreeQueryCount", "自定义查询 count：" + result);
				}
			} catch (Exception ex) {
				ex.printStackTrace();
			} finally {
				try {
					conn.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		} // IF OVER
		// 返回结果
		return result;
	}

	/** DateTime 类型转换 */
	private static SimpleDateFormat datetimeFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	/** Date 类型转换 */
	private static SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");

	/** Time 类型转换 */
	private static SimpleDateFormat timeFormat = new SimpleDateFormat("HH:mm:ss");

	@Override
	public FreeQueryResult getFreeQueryResult(String sql, String enterpriseId, Integer queryDataType, Integer desensiType) {
		//sql = prefixSQL(enterpriseId, sql);
		log.debug("getFreeQueryResult", "Execute 自定义查询来自 " + enterpriseId);
		log.debug("getFreeQueryResult", "自定义查询SQL：" + sql);
		FreeQueryResult result = new FreeQueryResult();
		Connection conn = MysqlHelper.getFreeQueryConnection(enterpriseId);
		if (conn != null) {
			try {
				Statement stat = conn.createStatement();
				ResultSet rs = stat.executeQuery(sql);
				while (rs.next()) {
					ResultSetMetaData metaData = rs.getMetaData();
					//log.debug("getFreeQueryResult", "查询结果列数：" + metaData.getColumnCount());
					for (int i = 0; i < metaData.getColumnCount(); i++) {
						String fieldName = metaData.getColumnName(i + 1);
						Object filedValue;
						if (queryDataType == QueryDataType.SAFE && FreeQueryServiceImpl.isFilterFields(desensiType, fieldName)) {//手机号和卡号
							filedValue = "******";
						}else if(queryDataType == QueryDataType.SAFE && FilterFieldUtils.FILETERS_USER_NAME.contains(fieldName)){//用户名
							filedValue = DecryptUtils.dataSecurityProcessUserName(rs.getString(fieldName));
						} else {
							switch (metaData.getColumnType(i + 1)) {
								case Types.TIMESTAMP:
									filedValue = rs.getTimestamp(fieldName) != null ? datetimeFormat.format(new Date(rs.getTimestamp(fieldName).getTime())) : "";
									break;
								case Types.DATE:
									filedValue = rs.getTimestamp(fieldName) != null ? dateFormat.format(new Date(rs.getTimestamp(fieldName).getTime())) : "";
									break;
								case Types.TIME:
									filedValue = rs.getTimestamp(fieldName) != null ? timeFormat.format(new Date(rs.getTimestamp(fieldName).getTime())) : "";
									break;
								default:
									filedValue = rs.getObject(fieldName);
							} // SWITCH OVER
						} // IF ELSE
						result.add(fieldName, filedValue);
					} // FOR OVER
				} // WHILE OVER
			} catch (Exception ex) {
				ex.printStackTrace();
			} finally {
				try {
					conn.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		} // IF OVER
		// 返回结果
		return result;
	}

	/** 生成下载任务
	 * @param userId
	 * @param name
	 * @param amount
	 * @param format
	 * @param sql
	 * @return
	 */
	public DownloadTask buildFreeQueryDownloadTask(String userId, String name, Integer amount, String format, String sql, String enterpriseId, Boolean useCompress, Integer queryDataType, String applyId, Integer desensiType) {
		//sql = prefixSQL(enterpriseId, sql);
		// 注册下载任务
		DownloadTask curTask = new DownloadTask();
		curTask.setId(String.valueOf((new Date()).getTime()));
		curTask.setQueryDataSource(QueryDataSource.FREE_QUERY); // 设置为自定义查询
		curTask.setUserId(userId);
		curTask.setName(name);
		curTask.setModuleName("自定义查询"); // FreeQuery 恒为自定义查询
		curTask.setAmount(amount);
		curTask.setFormat(format);
		curTask.setStartTime(new Date());
		curTask.setStatus(DownloadTaskStatus.WAITING);
		curTask.setUseCompress(useCompress ? Global.YES : Global.NO);
		// 新增下载任务属性
		curTask.setQueryDataType(queryDataType);
		curTask.setDesensiType(desensiType);
		curTask.setApplyId(applyId); // 设置申请编号
		if (queryDataType == QueryDataType.SAFE) { // 如果是安全模式
			curTask.setApplyPermitted(Global.YES); // 安全模式直接允许下载
			curTask.setApplyStatus(DownloadApplyStatus.PERMITTED_BY_SELF);
		} else { // 如果是完整模式
			curTask.setApplyPermitted(Global.NO); // 完整模式默认不允许
			curTask.setApplyStatus(DownloadApplyStatus.WAITING);
		} // IF ELSE OVER
		curTask.setRealSql(sql); // 自定义查询皆保存 SQL
		curTask.setEnterpriseId(enterpriseId);
		Integer preReportCount = DownloadTaskServiceImpl.getInstance().getDownloadTaskCountByEnterpriseId(curTask.getEnterpriseId());
		curTask.setReportId(ServiceUtils.buildReportId(curTask.getQueryDataType(), preReportCount));

		// 保存下载任务
		DownloadTaskServiceImpl.getInstance().createDownloadTask(curTask);

		// 生成条件
		FreeQueryTaskCondition condition = new FreeQueryTaskCondition();
		condition.setTaskId(curTask.getId());
		condition.setSql(sql);
		condition.setEnterpriseId(enterpriseId);
		condition.setDesensiType(desensiType);
		// 设置条件可否执行
		if (curTask.getApplyPermitted().equals(Global.YES)) {
			condition.setBuildPermitted(Global.YES);
		} else condition.setBuildPermitted(Global.NO);
		// 保存至列表
		this.taskConditions.add(condition);
		return curTask;
	}


	@Override
	@PostConstruct
	public void initTask() {
		try{
			List<DownloadRecord> list = this.downloadTaskService.listUnDownloadTask(QueryDataSource.FREE_QUERY);
			list = DataInitUtils.listByHost(list);
			log.debug("freeInitList", JSON.toJSONString(list));
			if(CollectionUtils.isNotEmpty(list)) {
				for (DownloadRecord record : list) {
					if(record.getDesensiType() != null){
						FreeQueryTaskCondition condition = new FreeQueryTaskCondition();
						condition.setTaskId(record.getId());
						condition.setSql(record.getRealSql());
						condition.setEnterpriseId(record.getEnterpriseId());
						condition.setDesensiType(record.getDesensiType());
						// 设置条件可否执行
						if (record.getApplyPermitted().equals(Global.YES)) {
							condition.setBuildPermitted(Global.YES);
						} else condition.setBuildPermitted(Global.NO);
						// 保存至列表
						this.taskConditions.add(condition);
					}
				}
			}
		}catch (Exception e){
			e.printStackTrace();
		}
	}

	@Override
	public List<String> listFilterPhoneField() {
		return FilterFieldUtils.FILTERS_PHONE_ONLY;
	}

	@Override
	public List<String> listFilterPhoneAndCardNoField() {
		return FilterFieldUtils.FILTERS_PHONE_AND_CARD;
	}

	@Override
	public List<String> listFilterUserName() {
		return FilterFieldUtils.FILETERS_USER_NAME;
	}

	private static SimpleDateFormat datetimeFormatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	private static SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");

	private static SimpleDateFormat timeFormatter = new SimpleDateFormat("HH:mm:ss");

	/** 下载任务执行计时器 */
	//private Timer downloadTaskTimer = new Timer();
	ScheduledExecutorService freeDownloadService = new ScheduledThreadPoolExecutor(1,
			new BasicThreadFactory.Builder().namingPattern("freeDownloadTimer-%d").daemon(true).build());

	/** 启动自助指标查询计划任务 */
	private void runDownloadTask(Integer interval) {
		freeDownloadService.scheduleAtFixedRate(new Runnable() {
			@Override
			public void run() {
				try {
					if (taskConditions != null && taskConditions.size() > 0) {
						//FreeQueryTaskCondition condition = taskConditions.remove(0); // 移除并获取第一个任务条件
						FreeQueryTaskCondition condition = null;
						for (int i = 0; i < taskConditions.size(); i++) {
							log.debug("自定义查询下载当前任务：", JSON.toJSONString(taskConditions.get(i)));
							if (taskConditions.get(i).getBuildPermitted().equals(Global.YES)) {
								condition = taskConditions.remove(i); // 移除并获取第一个任务条件
								break;
							} // IF OVER
						} // FOR OVER
						if (condition != null) {
							// 更新任务状态
							DownloadTask task = DownloadTaskServiceImpl.getInstance().getDownloadTaskById(condition.getTaskId());
							task.setStatus(DownloadTaskStatus.BUILDING);
							DownloadTaskServiceImpl.getInstance().updateDownloadTask(task);

							String fullQuery = condition.getSql();

							Connection conn = MysqlHelper.getFreeQueryConnection(condition.getEnterpriseId());
							if (conn != null) {
								try {
									// 设定为流式读取
									Statement stat = conn.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
									stat.setFetchSize(Integer.MIN_VALUE);
									ResultSet rs = stat.executeQuery(fullQuery);

									// 生成指定格式下载元文件
									String originalFilePath = "";
									if (task.getFormat().equals(DownloadFileFormat.CSV)) { // 如果指定为 CSV 格式
										log.debug("runDownloadTask.run", "准备生成自定义查询下载文件 " + condition.getTaskId() + ".csv");
										//String csvPath = SAVE_FOLDER + "/" + condition.getTaskId() + ".csv";
										originalFilePath = SAVE_FOLDER + "/" + condition.getTaskId() + ".csv";
										File tmp = new File(originalFilePath);
										if (tmp.exists()) { // 删除可能存在的文件
											tmp.delete();
										}

										//CSVWriter csvWriter = new CSVWriter(new FileWriter(csvPath), '\t');
										OutputStreamWriter out = new OutputStreamWriter(new FileOutputStream(originalFilePath), Charset.forName("GBK"));
										String queryDataType = task.getQueryDataType() == QueryDataType.FULL ? CsvDataFilterMode.NONE : CsvDataFilterMode.DESENSI;
										ResultSetHelper helper = new CsvResultSetHelper(queryDataType, FreeQueryServiceImpl.getFieldsFilters(condition.getDesensiType()), null, null, null);
										CSVWriter writer = new CSVWriter(out, ',');
										writer.setResultService(helper);
										writer.writeAll(rs, true);
										writer.close();
										out.close();
										log.debug("runDownloadTask.run", "已生成自定义查询下载文件 " + condition.getTaskId() + ".csv");
									} else {
										log.debug("runDownloadTask.run", "准备生成自定义查询下载文件 " + condition.getTaskId() + ".xlsx");
										originalFilePath = SAVE_FOLDER + "/" + condition.getTaskId() + ".xlsx";
										SXSSFWorkbook wb = new SXSSFWorkbook(100); // 内存中保留 100 行
										Sheet sheet = wb.createSheet();
										Row row = sheet.createRow(0);
										Cell cell;
										for (int j = 0; j < rs.getMetaData().getColumnCount(); ++j) { // 遍历创建表头
											String colName = rs.getMetaData().getColumnLabel(j + 1);
											cell = row.createCell(j);
											cell.setCellValue(colName);
										}
										// 遍历输出行
										int rowCount = 0;
										while (rs.next()) {
											rowCount++;
											row = sheet.createRow(rowCount);
											for (int j = 0; j < rs.getMetaData().getColumnCount(); ++j) {
												//String c = rs.getString(j + 1);
												//row.createCell(j).setCellValue(c);
												String cName = rs.getMetaData().getColumnName(j+1);
												if (task.getQueryDataType() == QueryDataType.SAFE
														&& (FreeQueryServiceImpl.isFilterFields(condition.getDesensiType(), cName) || FilterFieldUtils.FILETERS_USER_NAME.contains(cName))) {
													if(FilterFieldUtils.FILETERS_USER_NAME.contains(cName)){
														row.createCell(j).setCellValue(DecryptUtils.dataSecurityProcessUserName(rs.getString(j+1)));
													} else {
														row.createCell(j).setCellValue("******");
													}
												} else {
													int cType = rs.getMetaData().getColumnType(j + 1);
													switch (cType) {
														case Types.TIMESTAMP:
															row.createCell(j).setCellValue(rs.getTimestamp(j + 1) != null ? datetimeFormatter.format(rs.getTimestamp(j + 1)) : "");
															break;
														case Types.DATE:
															row.createCell(j).setCellValue(rs.getDate(j + 1) != null ? dateFormatter.format(rs.getDate(j + 1)) : "");
															break;
														case Types.TIME:
															row.createCell(j).setCellValue(rs.getTimestamp(j + 1) != null ? timeFormatter.format(rs.getTimestamp(j + 1)) : "");
															break;
														default:
															row.createCell(j).setCellValue(rs.getString(j + 1));
															break;
													}
												}
											} // FOR OVER
										} // WHILE OVER
										FileOutputStream fileOut = new FileOutputStream(originalFilePath);
										wb.write(fileOut);
										//fileOut.flush(); // SXSSFWorkbook 使用 auto-flush 模式
										fileOut.close();
										//wb.close();
										wb.dispose(); // SXSSFWorkbook 没有 close 方法
										log.debug("runDownloadTask.run", "已生成自定义查询下载文件 " + condition.getTaskId() + ".xlsx");
									}
									String cloudFileUrl = "";
									if (task.getUseCompress().equals(Global.YES)) {
										log.debug("runDownloadTask.run", "准备生成自定义查询压缩文件 " + condition.getTaskId() + ".zip");
										String zipFilePath = SAVE_FOLDER + "/" + condition.getTaskId() + ".zip";
										File zipFile = new File(zipFilePath);
										ZipOutputStream zos = null;
										byte[] buf = new byte[1024];
										int length = 0;
										try {
											OutputStream os = new FileOutputStream(zipFilePath);
											BufferedOutputStream bos = new BufferedOutputStream(os);
											zos = new ZipOutputStream(bos);
											zos.setLevel(6); // 压缩率选择 0-9
											InputStream is = new FileInputStream(originalFilePath);
											BufferedInputStream bis = new BufferedInputStream(is);
											zos.putNextEntry(new ZipEntry(originalFilePath.substring(originalFilePath.lastIndexOf("/") + 1)));
											while ((length = bis.read(buf)) > 0) {
												zos.write(buf, 0, length);
											}
											bis.close();
											is.close();
											//bos.close();
											//os.close();
											log.debug("runDownloadTask.run", "已生成自定义查询压缩文件 " + condition.getTaskId() + ".zip");
										} catch (Exception ex2) {
											throw ex2;
										} finally {
											zos.closeEntry();
											zos.close();
										}
										cloudFileUrl = FileUploadUtil.simpleUploadFileFromLocal(zipFile, task.getId(), BucketNameEnum.COMPRESS_60000.getName());
									} else {
										cloudFileUrl = FileUploadUtil.simpleUploadFileFromLocal(new File(originalFilePath), task.getId(), BucketNameEnum.REPORT_50000.getName());
									}
									log.debug("上传腾讯云", "地址为："+cloudFileUrl);
									task.setStatus(DownloadTaskStatus.COMPLISHED);
									task.setOverTime(new java.util.Date());

									String taskFileExt = task.getUseCompress().equals(Global.YES) ? ".zip" : task.getFormat().equals(DownloadFileFormat.CSV) ? ".csv" : ".xlsx";
									task.setFilePath(cloudFileUrl);
									DownloadTaskServiceImpl.getInstance().updateDownloadTask(task);

								} catch (Exception ex) {
									ex.printStackTrace();
									// 标记任务异常
									task.setStatus(DownloadTaskStatus.ERROR);
									task.setOverTime(new Date());
									DownloadTaskServiceImpl.getInstance().updateDownloadTask(task);
								} finally {
									try {
										conn.close();
									} catch (SQLException e) {
										e.printStackTrace();
									}
								}
							} // IF OVER
						} // IF OVER
					} // 没有任务则忽略
				}catch (Exception e){
					log.debug("自定义查询下载异常", e.getMessage());
					e.printStackTrace();
				}
			}
		}, interval*1000, interval*1000, TimeUnit.MILLISECONDS);
	}

	/** 下载申请检查计时器 */
	//private Timer applyTimer = new Timer();

	ScheduledExecutorService freeApplyService = new ScheduledThreadPoolExecutor(1,
			new BasicThreadFactory.Builder().namingPattern("freeApplyTimer-%d").daemon(true).build());

	/** 运行下载申请任务
	 * @param interval
	 */
	private void runApplyTask(Integer interval) {
		Config config = ConfigService.getConfig("application");
		final String url = config.getProperty("applyUrl", "");
		this.freeApplyService.scheduleAtFixedRate(new Runnable() {
			@Override
			public void run() {
				List<DownloadTask> waitingTasks = DownloadTaskServiceImpl.getInstance().getDownloadTaskOfWaiting(QueryDataSource.FREE_QUERY);
				waitingTasks = DataInitUtils.listTaskByHost(waitingTasks);
				for (DownloadTask task : waitingTasks) {
					log.debug("自定义查询任务审批状态变更：", JSON.toJSONString(task));
					if (!task.getApplyId().equals("")) {
//						String applyStatusText = HttpUtils.doGet("http://www.gicdev.com/api-admin/apply-info?type=2&applyId=" + task.getApplyId());
						String applyStatusText = HttpUtils.doGet(url + task.getApplyId());
						log.debug("runApplyTask.run", "查询自助指标查询 " + task.getId() + " 审核状态码" + applyStatusText);
						if(StringUtils.isBlank(applyStatusText)){
							break;
						}
						boolean conditionRemove = false; // 从上传条件移除
						boolean conditionUpdate = false; // 更新条件
						if (applyStatusText.indexOf("\"checkStatus\":0") > 0) { // 如果自审核通过
							task.setApplyStatus(DownloadApplyStatus.PERMITTED_BY_SELF);
							task.setApplyPermitted(Global.YES);
							conditionUpdate = true;
						} else if (applyStatusText.indexOf("\"checkStatus\":2") > 0) { // 如果第三方审核通过
							task.setApplyStatus(DownloadApplyStatus.PERMITTED_BY_OTHER);
							task.setApplyPermitted(Global.YES);
							conditionUpdate = true;
						} else if (applyStatusText.indexOf("\"checkStatus\":1") > 0) { // 如果审核超时
							task.setApplyStatus(DownloadApplyStatus.TIMEOUT);
							task.setApplyPermitted(Global.NO);
							conditionRemove = true;
						}
						if (conditionRemove || conditionUpdate) {
							DownloadTaskServiceImpl.getInstance().updateDownloadTask(task);
							log.debug("runApplyTask.run", "自定义查询 " + task.getId() + " 审核状态已更新：" + task.getApplyStatus() + " / " + task.getApplyPermitted());
						}
						if (conditionUpdate) { // 如果可以更新状态
							DownloadTaskServiceImpl.getInstance().updateDownloadTask(task);
							if (taskConditions != null && taskConditions.size() > 0) {
								for (FreeQueryTaskCondition condition : taskConditions) {
									if (condition.getTaskId().equals(task.getId())) {
										condition.setBuildPermitted(Global.YES);
										log.debug("runApplyTask.run", "自定义查询 " + task.getId() + " 已允许生成");
										break;
									} // IF OVER
								} // FOR OVER
							} // IF OVER
						} else if (conditionRemove) { // 如果可以移除
							DownloadTaskServiceImpl.getInstance().updateDownloadTask(task);
							if (taskConditions != null && taskConditions.size() > 0) {
								for (int j=0; j<taskConditions.size(); j++) {
									if (taskConditions.get(j).getTaskId().equals(task.getId())) {
										taskConditions.remove(j);
										log.debug("runApplyTask.run", "自定义指标查询 " + task.getId() + " 已从待生成队列移除");
									} // IF OVER
								} // FOR OVER
							} // IF OVER
						} // IF ELSE OVER
					}
				}
			}
		}, interval*1000, interval*1000, TimeUnit.MILLISECONDS);
	}


	public static void main(String[] args){
		long startTs = (System.currentTimeMillis() / 60*1000 + 1) * 60*1000 - System.currentTimeMillis();
		System.out.println("startTs = " + startTs);
	}
}
